% ==========================================================
% 
%           Neural Networks A Classroom Approach
%                     Satish Kumar
%             Copyright Tata McGraw Hill, 2004
%
%     MATLAB code that implements the SVM for linearly
%     separable data
%               Reference: Table 8.1;Page 283
%
% ==========================================================

X=[0.5 0.5					% Data points
   0.5 1.0
   1.  1.5
   1.5 0.5
   1.5 2.0
   2.0 1.0
   2.5 2.0
   2.5 2.5];
 
D=[-1 -1 -1 -1 1 1 1 1]'; % Corresponding classes

q = size(X,1);				% Size of data set
C = 5;

% Threshold for checking support vectors
if C==Inf 
  epsilon = 1e-5;
else
  epsilon = C*1e-6;
end
    
H = zeros(q,q);			% Initialize Hessian matrix  
for i=1:q					% Set up the Hessian
   for j=1:q
      H(i,j) = D(i)*D(j)*X(i,:)*X(j,:)'; 
   end
end
f = -ones(q,1);  
    
% Add small amount of zero order regularisation to 
% avoid problems when Hessian is badly conditioned. 
H = H+1e-10*eye(size(H));
    
% Parameters for the Optimization problem
numeqconstraints = 1;	% Number of equality constraints = 1;
A = D';						% Set up the equality constraint
b = 0;
vlb = zeros(q,1);			% Lower bound of lambdas = 0
vub = C*ones(q,1);		% No upper bound
x0 = zeros(q,1);			% Initial point is 0

% Invoke MATLAB's standard qp function for quadratic optimization
[alpha lambda how] = qp(H, f, A, b, vlb, vub, x0, numeqconstraints);
    
svindex = find( alpha > epsilon);	% Support Vector indices
ns = length(svindex);					% Number of support vectors
svii = find( alpha > epsilon & alpha < (C - epsilon)); %unbounded
if length(svii) > 0
  b0 =  (1/length(svii))*sum(D(svii) - H(svii,svindex)*alpha(svindex).*D(svii));
else 
  fprintf('No support vectors on margin - cannot compute bias.\n');
end
  
% Scale the axes
xmin = 0;
xmax = 3; 
ymin = 0;
ymax = 3; 
    
% Plot function value
[x,y] = meshgrid(xmin:(xmax-xmin)/50:xmax,ymin:(ymax-ymin)/50:ymax); 
z = b0*ones(size(x));

for x1 = 1 : size(x,1)
   for y1 = 1 : size(x,2) 
      input(1) = x(x1,y1);
      input(2) = y(x1,y1);
      for i = 1 : length(D)
         if (abs(alpha(i)) > epsilon)
    	     z(x1,y1) = z(x1,y1) + D(i)*alpha(i)*input*X(i,:)'; 
         end
      end
   end
end
sp = pcolor(x,y,z);
shading interp;
colormap(bone);
    
hold on
for i = 1:size(D)
   if (D(i) == 1)
     h(1) = plot(X(i,1),X(i,2),'k^', 'MarkerSize', 8); % Class A
   else
     h(2) = plot(X(i,1),X(i,2),'w*', 'MarkerSize', 8); % Class B
   end
   if (abs(alpha(i)) > epsilon)
     plot(X(i,1),X(i,2),'wo', 'MarkerSize', 12) % Support Vector
   end
end 

% Plot Boundary contour
hold on
contour(x,y,z,[0 0],'w')
contour(x,y,z,[-1 -1],'w:')
contour(x,y,z,[1 1],'w:')
hold off
axis([0 3 0 3])
    
    
